@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-close-confirmation-dialog.c 5345 2006-12-07 23:33:28Z paolo $
+ * $Id: gedit-close-confirmation-dialog.c 6803 2009-01-03 16:11:03Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -511,11 +511,15 @@ build_single_doc_dialog (GeditCloseConfirmationDialog *dlg)
 	doc_name = gedit_document_get_short_name_for_display (doc);
 
 	if (dlg->priv->disable_save_to_disk)
-		str = g_strdup_printf (_("Changes to document \"%s\" will be permanently lost."),
-				       doc_name);
+	{
+		str = g_markup_printf_escaped (_("Changes to document \"%s\" will be permanently lost."),
+					       doc_name);
+	}
 	else
-		str = g_strdup_printf (_("Save the changes to document \"%s\" before closing?"),
-				       doc_name);
+	{
+		str = g_markup_printf_escaped (_("Save changes to document \"%s\" before closing?"),
+					       doc_name);
+	}
 
 	g_free (doc_name);
 
