@@ -26,7 +26,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-preferences-dialog.c 6727 2008-12-25 21:02:48Z pborelli $
+ * $Id: gedit-preferences-dialog.c 6876 2009-02-03 18:02:18Z icq $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -917,7 +917,7 @@ uninstall_scheme_clicked (GtkButton              *button,
 			if (new_iter_set)
 				gtk_tree_model_get (model, &new_iter,
 						    ID_COLUMN, &new_id,
-						    NULL);
+						    -1);
 						    			
 			real_new_id = populate_color_scheme_list (dlg, new_id);
 			g_free (new_id);
