@@ -218,9 +218,11 @@ gedit_document_saver_new (GeditDocument       *doc,
 
 	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), NULL);
 
+#ifndef G_OS_WIN32
 	if (gedit_utils_uri_has_file_scheme (uri))
 		saver_type = GEDIT_TYPE_LOCAL_DOCUMENT_SAVER;
 	else
+#endif
 		saver_type = GEDIT_TYPE_GIO_DOCUMENT_SAVER;
 
 	if (encoding == NULL)
@@ -334,7 +336,9 @@ gedit_document_saver_write_document_contents (GeditDocumentSaver  *saver,
 
 	/* Truncate the file to 0, in case it was not empty */
 	if (res)
+	{
 		res = (ftruncate (fd, 0) == 0);
+	}
 
 	/* Save the file content */
 	if (res)
@@ -381,7 +385,17 @@ gedit_document_saver_write_document_contents (GeditDocumentSaver  *saver,
 		}
 	}
 
-	g_free (contents);
+#ifdef HAVE_FSYNC
+	/* Ensure that all the data reaches disk */
+	if (res && fsync (fd) != 0)
+	{
+		g_set_error (error,
+			     G_IO_ERROR,
+			     g_io_error_from_errno (errno),
+			     "%s", g_strerror (errno));
+		res = FALSE;
+	}
+#endif
 
 	if (!res)
 	{
@@ -391,6 +405,8 @@ gedit_document_saver_write_document_contents (GeditDocumentSaver  *saver,
 			     "%s", g_strerror (errno));
 	}
 
+	g_free (contents);
+
 	return res;
 }
 
