@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-window-private.h 6755 2008-12-29 18:52:22Z jessevdk $
+ * $Id: gedit-window-private.h 6865 2009-01-24 18:02:20Z pborelli $
  */
 
 #ifndef __GEDIT_WINDOW_PRIVATE_H__
@@ -49,12 +49,23 @@ struct _GeditWindowPrivate
 	GtkWidget      *hpaned;
 	GtkWidget      *vpaned;
 	
+	GtkWidget      *tab_width_combo;
+	GtkWidget      *language_combo;
+	
 	GeditMessageBus *message_bus;	
 
+	/* Widgets for fullscreen mode */
+	GtkWidget      *fullscreen_controls;
+	guint           fullscreen_animation_timeout_id;
+	gboolean        fullscreen_animation_enter;
+
 	/* statusbar and context ids for statusbar messages */
 	GtkWidget      *statusbar;	
 	guint           generic_message_cid;
 	guint           tip_message_cid;
+	guint 		tab_width_id;
+	guint 		spaces_instead_of_tabs_id;
+	guint 		language_changed_id;
 
 	/* Menus & Toolbars */
 	GtkUIManager   *manager;
@@ -87,6 +98,8 @@ struct _GeditWindowPrivate
 
 	GeditWindowState state;
 
+	gint		bottom_panel_item_removed_handler_id;
+
 	GtkWindowGroup *window_group;
 
 	gchar          *default_path;
