@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit.c 6634 2008-11-29 17:50:59Z pborelli $
+ * $Id: gedit.c 6912 2009-02-22 17:00:21Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -48,6 +48,7 @@
 #include "gedit-app.h"
 #include "gedit-commands.h"
 #include "gedit-debug.h"
+#include "gedit-dirs.h"
 #include "gedit-encodings.h"
 #include "gedit-metadata-manager.h"
 #include "gedit-plugins-engine.h"
@@ -56,12 +57,32 @@
 #include "gedit-utils.h"
 #include "gedit-window.h"
 
-#include "bacon-message-connection.h"
 #include "eggsmclient.h"
 #include "eggdesktopfile.h"
 
+#ifdef G_OS_WIN32
+#define SAVE_DATADIR DATADIR
+#undef DATADIR
+#include <windows.h>
+#define DATADIR SAVE_DATADIR
+#undef SAVE_DATADIR
+#endif
+
 static guint32 startup_timestamp = 0;
+
+#ifndef G_OS_WIN32
+#include "bacon-message-connection.h"
+
 static BaconMessageConnection *connection;
+#endif
+
+static void
+show_version_and_quit (void)
+{
+	g_print ("%s - Version %s\n", g_get_application_name (), VERSION);
+
+	exit (0);
+}
 
 /* command line */
 static gint line_position = 0;
@@ -74,6 +95,9 @@ static GSList *file_list = NULL;
 
 static const GOptionEntry options [] =
 {
+	{ "version", 'V', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK,
+	  show_version_and_quit, N_("Show the application's version"), NULL },
+
 	{ "encoding", '\0', 0, G_OPTION_ARG_STRING, &encoding_charset,
 	  N_("Set the character encoding to be used to open the files listed on the command line"), N_("ENCODING")},
 
@@ -108,16 +132,10 @@ gedit_get_command_line_data (void)
 			}
 			else
 			{
-				gchar *canonical_uri;
-				
-				canonical_uri = gedit_utils_make_canonical_uri_from_shell_arg (remaining_args[i]);
-				
-				if (canonical_uri != NULL)
-					file_list = g_slist_prepend (file_list, 
-								     canonical_uri);
-				else
-					g_print (_("%s: malformed file name or URI.\n"),
-						 remaining_args[i]);
+				GFile *file;
+
+				file = g_file_new_for_commandline_arg (remaining_args[i]);
+				file_list = g_slist_prepend (file_list, file);
 			} 
 		}
 
@@ -172,6 +190,7 @@ get_startup_timestamp ()
 	return (retval > 0) ? retval : 0;
 }
 
+#ifndef G_OS_WIN32
 static GdkDisplay *
 display_open_if_needed (const gchar *name)
 {
@@ -266,7 +285,13 @@ on_message_received (const char *message,
 			uris = g_strsplit (params[4], " ", n_uris);
 
 			for (j = 0; j < n_uris; j++)
-				file_list = g_slist_prepend (file_list, uris[j]);
+			{
+				GFile *file;
+
+				file = g_file_new_for_uri (uris[j]);
+				file_list = g_slist_prepend (file_list, file);
+			}
+
 			file_list = g_slist_reverse (file_list);
 
 			/* the list takes ownerhip of the strings,
@@ -340,7 +365,7 @@ on_message_received (const char *message,
 	gtk_window_present (GTK_WINDOW (window));
 
 	/* free the file list and reset to default */
-	g_slist_foreach (file_list, (GFunc) g_free, NULL);
+	g_slist_foreach (file_list, (GFunc) g_object_unref, NULL);
 	g_slist_free (file_list);
 	file_list = NULL;
 
@@ -432,9 +457,14 @@ send_bacon_message (void)
 
 		for (l = file_list; l != NULL; l = l->next)
 		{
-			command = g_string_append (command, l->data);
+			gchar *uri;
+
+			uri = g_file_get_uri (G_FILE (l->data));
+			command = g_string_append (command, uri);
 			if (l->next != NULL)
 				command = g_string_append_c (command, ' ');
+
+			g_free (uri);
 		}
 	}
 
@@ -445,6 +475,37 @@ send_bacon_message (void)
 
 	g_string_free (command, TRUE);
 }
+#endif /* G_OS_WIN32 */
+
+#ifdef G_OS_WIN32
+static void
+setup_path (void)
+{
+	/* Set PATH to include the gedit executable's folder */
+	wchar_t exe_filename[MAX_PATH];
+	wchar_t *p;
+	gchar *exe_folder_utf8;
+	gchar *path;
+	
+	GetModuleFileNameW (NULL, exe_filename, G_N_ELEMENTS (exe_filename)); 
+	
+	p = wcsrchr (exe_filename, L'\\');
+	g_assert (p != NULL);
+	
+	*p = L'\0';
+	exe_folder_utf8 = g_utf16_to_utf8 (exe_filename, -1, NULL, NULL, NULL);
+	
+	path = g_build_path (";",
+			     exe_folder_utf8,
+			     g_getenv ("PATH"),
+			     NULL);
+	if (!g_setenv ("PATH", path, TRUE))
+		g_warning ("Could not set PATH for gedit");
+	
+	g_free (exe_folder_utf8);
+	g_free (path);
+}
+#endif
 
 int
 main (int argc, char *argv[])
@@ -455,6 +516,8 @@ main (int argc, char *argv[])
 	GeditApp *app;
 	gboolean restored = FALSE;
 	GError *error = NULL;
+	gchar *dir;
+	gchar *icon_dir;
 
 	/* Init glib threads asap */
 	g_thread_init (NULL);
@@ -465,7 +528,9 @@ main (int argc, char *argv[])
 	
 	setlocale (LC_ALL, "");
 
-	bindtextdomain (GETTEXT_PACKAGE, GEDIT_LOCALEDIR);
+	dir = gedit_dirs_get_gedit_locale_dir ();
+	bindtextdomain (GETTEXT_PACKAGE, dir);
+	g_free (dir);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
@@ -477,6 +542,10 @@ main (int argc, char *argv[])
 	g_option_context_add_group (context, gtk_get_option_group (FALSE));
 	g_option_context_add_group (context, egg_sm_client_get_option_group ());
 
+#ifdef G_OS_WIN32
+	setup_path ();
+#endif
+
 	gtk_init (&argc, &argv);
 
 	if (!g_option_context_parse (context, &argc, &argv, &error))
@@ -489,6 +558,7 @@ main (int argc, char *argv[])
 
 	g_option_context_free (context);
 
+#ifndef G_OS_WIN32
 	gedit_debug_message (DEBUG_APP, "Create bacon connection");
 
 	connection = bacon_message_connection_new ("gedit");
@@ -525,15 +595,27 @@ main (int argc, char *argv[])
 	{
 		g_warning ("Cannot create the 'gedit' connection.");
 	}
+#endif
 
 	gedit_debug_message (DEBUG_APP, "Set icon");
 	
+	dir = gedit_dirs_get_gedit_data_dir ();
+	icon_dir = g_build_filename (dir,
+				     "icons",
+				     NULL);
+	g_free (dir);
+	
 	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
-					   GEDIT_ICONDIR);
+					   icon_dir);
+	g_free (icon_dir);
 
+#if !defined PLATFORM_OSX && !defined G_OS_WIN32
 	/* Set the associated .desktop file */
-#ifndef PLATFORM_OSX
 	egg_set_desktop_file (DATADIR "/applications/gedit.desktop");
+#else
+	/* manually set name and icon */
+	g_set_application_name("gedit");
+	gtk_window_set_default_icon_name ("accessories-text-editor");
 #endif
 
 	/* Load user preferences */
@@ -581,7 +663,7 @@ main (int argc, char *argv[])
 		gedit_debug_message (DEBUG_APP, "Show window");
 		gtk_widget_show (GTK_WIDGET (window));
 
-		g_slist_foreach (file_list, (GFunc) g_free, NULL);
+		g_slist_foreach (file_list, (GFunc) g_object_unref, NULL);
 		g_slist_free (file_list);
 		file_list = NULL;
 
@@ -596,7 +678,9 @@ main (int argc, char *argv[])
 	gedit_debug_message (DEBUG_APP, "Start gtk-main");		
 	gtk_main();
 
+#ifndef G_OS_WIN32
 	bacon_message_connection_free (connection);
+#endif
 
 	/* We kept the original engine reference here. So let's unref it to
 	 * finalize it properly. 
