@@ -558,13 +558,8 @@ on_selection_changed_cb (GtkTreeSelection *selection,
 	if (!GEDIT_IS_FILE_BROWSER_STORE (model))
 		return;
 	
-	if (gedit_file_browser_widget_get_num_selected_files_or_directories (data->tree_widget) > 1)
-		sensitive = FALSE;
-	else if (!gedit_file_browser_widget_get_selected_directory (data->tree_widget, &iter))
-		sensitive = FALSE;
-	else
-		sensitive = TRUE;
-	
+	sensitive = gedit_file_browser_widget_get_selected_directory (data->tree_widget, &iter);
+
 	if (sensitive) {
 		gtk_tree_model_get (model, &iter, 
 				    GEDIT_FILE_BROWSER_STORE_COLUMN_URI, 
@@ -601,7 +596,7 @@ static GtkActionEntry extra_actions[] =
 };
 
 static GtkActionEntry extra_single_selection_actions[] = {
-	{"OpenTerminal", GTK_STOCK_EXECUTE, N_("_Open terminal here"),
+	{"OpenTerminal", "utilities-terminal", N_("_Open terminal here"),
 	 NULL,
 	 N_("Open a terminal at the currently opened directory"),
 	 G_CALLBACK (on_action_open_terminal)}
@@ -690,9 +685,7 @@ impl_activate (GeditPlugin * plugin, GeditWindow * window)
 	GeditFileBrowserStore * store;
 	gchar *data_dir;
 
-	data = g_new (GeditFileBrowserPluginData, 1);
-	data->auto_root = FALSE;
-	data->end_loading_handle = 0;
+	data = g_new0 (GeditFileBrowserPluginData, 1);
 	
 	data_dir = gedit_plugin_get_data_dir (plugin);
 	data->tree_widget = GEDIT_FILE_BROWSER_WIDGET (gedit_file_browser_widget_new (data_dir));
