@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: gedit-spell-plugin.c 6741 2008-12-28 19:23:36Z icq $
+ * $Id: gedit-spell-plugin.c 6779 2009-01-01 15:41:34Z icq $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -72,7 +72,7 @@ static const GtkActionEntry action_entries[] =
 {
 	{ "CheckSpell",
 	  GTK_STOCK_SPELL_CHECK,
-	  N_("_Check Spelling"),
+	  N_("_Check Spelling..."),
 	  "<shift>F7",
 	  N_("Check the current document for incorrect spelling"),
 	  G_CALLBACK (spell_cb)
